@@ -58,15 +58,10 @@ ValidateInputs <- function(ref, labels, refType) {
     
   }
   
-  if (all(c("ont", "label", "sample", "dataset") %in% colnames(labels))) {
+  if (all(colnames(labels) %in% c("ont", "label", "sample", "dataset"))) {
     labels <- labels[, c("ont", "label", "sample", "dataset")]
   } else {
-    if (inherits(ref, "SummarizedExperiment") || inherits(ref, "SingleCellExperiment")) {
-      stop("colData(ref) must have 4 columns: 'ont'', 'label'', 'sample'' and 'dataset'")
-      
-    }else{
-      stop("labels must have 4 columns: 'ont'', 'label'', 'sample'' and 'dataset'")
-    }
+    stop("labels must have 4 columns: 'ont'', 'label'', 'sample'' and 'dataset'")
   }
   
   if (length(unique(labels$label)) < 3) {
@@ -83,10 +78,10 @@ ValidateInputs <- function(ref, labels, refType) {
 }
 
 # Function to convert single-cell data to pseudo-bulk data
-ScToPseudoBulk <- function(ref, labels, minPbCells, minPbSamples, BPPARAM) {
+ScToPseudoBulk <- function(ref, labels, minPbCells, minPbSamples) {
   cellTypes <- unique(labels$label)
   
-  groupsList <- BiocParallel::bplapply(cellTypes, function(cellType) {
+  groupsList <- lapply(cellTypes, function(cellType) {
     cellTypeSamples <- labels[labels$label == cellType, ]$sample
     
     # Calculate maximum possible number of groups given minPbCells
@@ -109,7 +104,7 @@ ScToPseudoBulk <- function(ref, labels, minPbCells, minPbSamples, BPPARAM) {
                 / (length(cellTypeSamplesShuffled) / numGroups))
       )
       
-      tmp <- vapply(listOfShuffledSamples, function(group) {
+      vapply(listOfShuffledSamples, function(group) {
         if (length(group) == 1) {
           ref[, group]
         } else {
@@ -122,27 +117,24 @@ ScToPseudoBulk <- function(ref, labels, minPbCells, minPbSamples, BPPARAM) {
       }, FUN.VALUE = double(nrow(ref)))
     } else {
       tmp <- ref[, cellTypeSamples]
+      colnames(tmp) <- as.character(seq_along(colnames(tmp)))
       tmp <- as.data.frame(as.matrix(tmp))
+      tmp
     }
-    
-    colnames(tmp) <- paste0(cellType, ".", seq_len(ncol(tmp)))
-    tmp
-    
-  }, BPPARAM = BPPARAM)
+  })
   names(groupsList) <- cellTypes
   
-  pseudoRef <- as.matrix(Reduce(cbind, groupsList))
+  pseudoRef <- as.matrix(dplyr::bind_cols(groupsList))
   rownames(pseudoRef) <- rownames(ref)
   
-  label_tmp <- tibble::tibble(
-    label = sub("\\.\\d+$", "", colnames(pseudoRef)),
-    sample = colnames(pseudoRef),
-    dataset = "pseudoBulk"
-  )
-  
-  pseudoLabel <- label_tmp %>%
-    left_join(unique(labels[,c("ont", "label")]), by = "label") %>%
-    dplyr::select(ont, everything()) %>%
+  pseudoLabel <- tibble::tibble(labels) %>%
+    dplyr::select("ont", "label") %>%
+    unique() %>%
+    dplyr::right_join(tibble::tibble(
+      label = sub("\\.\\d+$", "", colnames(pseudoRef)),
+      sample = colnames(pseudoRef),
+      dataset = "pseudoBulk"
+    ), by = "label") %>%
     as.data.frame()
   
   return(list(ref = pseudoRef, labels = pseudoLabel))
@@ -637,8 +629,7 @@ LearnParams <- function(gepMat, corMat, signatures, depList, BPPARAM) {
 #' It supports references derived from RNA-Seq, microarray, and scRNA-Seq data and can be derived from various tissues and organisms. 
 #'
 #' @importFrom magrittr %>%
-#' @importFrom tidyselect everything
-#' @importFrom dplyr bind_cols select right_join mutate_all pull bind_rows left_join select
+#' @importFrom dplyr bind_cols select right_join mutate_all pull bind_rows
 #' @importFrom tibble tibble
 #' @importFrom readr read_tsv
 #' @importFrom BiocParallel SerialParam MulticoreParam bplapply
@@ -665,7 +656,6 @@ LearnParams <- function(gepMat, corMat, signatures, depList, BPPARAM) {
 #'   
 #'   \strong{Notes:}
 #'   \itemize{
-#'     \item If is RNA-Seq data - normalization by gene length is highly recommended.
 #'     \item If multiple assays exist, \code{"tpm"} is prioritized.
 #'     \item For microarray data, the \code{"counts"} assay must be used.
 #'   }
@@ -741,12 +731,6 @@ LearnParams <- function(gepMat, corMat, signatures, depList, BPPARAM) {
 #' dice_labels$dataset <- "DICE"
 #' DICE.xCell2Ref <- xCell2::xCell2Train(ref = dice_ref, labels = dice_labels, refType = "rnaseq")
 #'
-#' # Parallel processing example with BiocParallel
-#' library(BiocParallel)
-#' parallel_param <- MulticoreParam(workers = 2)
-#' DICE.xCell2Ref <- xCell2::xCell2Train(ref = dice_ref, labels = dice_labels, refType = "rnaseq",
-#'  BPPARAM = parallel_param)
-#' 
 #' @seealso 
 #' \code{\link{xCell2Analysis}}, for enrichment analysis. 
 #' \code{\link{xCell2GetLineage}}, for refining cell type dependencies.
@@ -777,7 +761,7 @@ xCell2Train <- function(ref,
   # Generate pseudo bulk from scRNA-Seq reference
   if (refType == "sc") {
     message("Converting scRNA-seq reference to pseudo bulk...")
-    pbData <- ScToPseudoBulk(ref, labels, minPbCells, minPbSamples, BPPARAM)
+    pbData <- ScToPseudoBulk(ref, labels, minPbCells, minPbSamples)
     ref <- pbData$ref
     labels <- pbData$labels
   }
